#version 130

#include "settings.glsl"

varying vec2 TexCoords;
uniform sampler2D colortex4;

float ab = CHROMATIC_ABERRATION;
float zoom = ZOOM;
float strength = DIST_STRENGTH;
float vignetteStrength = VIGNETTE_STRENGTH;
float vignetteRadius = VIGNETTE_RADIUS;
float stripeWidth = BLACK_STRIPES_WIDTH;
float stripeEdgeSoftness = BLACK_STRIPES_SOFT;

void main() {
	vec3 color = texture2DLod(colortex4, TexCoords, 0).rgb;

    // Apply zoom
    vec2 centeredCoord = (TexCoords - 0.5) * zoom + 0.5;
    float distance = length(centeredCoord - 0.5);

    // Fisheye effect
    vec2 fisheyeUV = (centeredCoord - 0.5) * (1.0 + strength * distance * distance) + 0.5;

	// Chromatic aberration
    vec2 redUV = fisheyeUV + vec2(ab, ab) * distance;
    vec2 greenUV = fisheyeUV;
    vec2 blueUV = fisheyeUV + vec2(-ab, -ab) * distance;
    color.r = texture2D(colortex4, redUV).r;
    color.g = texture2D(colortex4, greenUV).g;
    color.b = texture2D(colortex4, blueUV).b;

    // Apply saturation, brightness, and contrast
    float average = (color.r + color.g + color.b) / 3.0;
    color.rgb = mix(vec3(average), color.rgb, SATURATION);
    color.rgb += BRIGHTNESS;
    color.rgb = ((color.rgb - 0.5) * CONTRAST) + 0.5;

    #if VIGNETTE == 1
    float vignette = smoothstep(vignetteRadius, vignetteRadius + vignetteStrength, distance);
    color = mix(color, vec3(0.0), vignette);
    #endif

    #if BLACK_STRIPES == 1
    float leftStripe = smoothstep(stripeWidth, stripeWidth - stripeEdgeSoftness, TexCoords.x);
    float rightStripe = smoothstep(1.0 - stripeWidth, 1.0 - (stripeWidth - stripeEdgeSoftness), TexCoords.x);
    float stripeEffect = max(leftStripe, rightStripe);
    color = mix(color, vec3(0.0), stripeEffect);
	#endif

	gl_FragColor = vec4(color, 1.0);
}
